/** 
 * @file microdia.h
 * @author Nicolas VIVIEN
 * @date 2008-02-01
 * @version v0.0.0
 *
 * @brief Driver for Microdia USB video camera
 *
 * @note Copyright (C) Nicolas VIVIEN
 *
 * @par Licences
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/usb.h>
#include <media/v4l2-common.h>

#ifndef MICRODIA_H
#define MICRODIA_H


#define DRIVER_NAME			"microdia"			/**< Name of this driver */
#define DRIVER_VERSION			"v0.0.0"			/**< Version of this driver */
#define DRIVER_VERSION_NUM		0x000000			/**< Version numerical of this driver */
#define DRIVER_DESC			"Microdia USB Video Camera"	/**< Short description of this driver */
#define DRIVER_AUTHOR			"Nicolas VIVIEN"		/**< Author of this driver */
#define PREFIX				DRIVER_NAME ": "		/**< Prefix use for the MICRODIA "printk" */

#define USB_VENDOR_ID_MICRODIA		0x0c45				/**< Vendor ID of MICRODIA */
#define USB_VENDOR_ID_MICROSOFT		0x045e				/**< Vendor ID of MICROSOFT */
#define USB_VENDOR_ID_CHICONY		0x04f2				/**< Vendor ID of CHICONY */
#define USB_VENDOR_ID_TRUST		0x145f				/**< Vendor ID of TRUST */

// SN9C201:
#define USB_UDIA_MICRODIA_PRODUCT_ID_6240	0x6240			/**< SN9C201 + MI1300 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6242	0x6242			/**< SN9C201 + MI1310 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6243	0x6243			/**< SN9C201 + S5K4AAFX */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6248	0x6248			/**< SN9C201 + OV9655 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_624B	0x624b			/**< SN9C201 + CX1332 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_624C	0x624c			/**< SN9C201 + MI1320 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_624E	0x624e			/**< SN9C201 + SOI968 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_624F	0x624f			/**< SN9C201 + OV9650 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6253	0x6253			/**< SN9C201 + OV9650 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6260	0x6260			/**< SN9C201 + OV7670ISP */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6262	0x6262			/**< SN9C201 + OM6802 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6270	0x6270			/**< SN9C201 + MI0360/MT9V111 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_627A	0x627a			/**< SN9C201 + S5K53BEB */
#define USB_UDIA_MICRODIA_PRODUCT_ID_627B	0x627b			/**< SN9C201 + OV7660 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_627C	0x627c			/**< SN9C201 + HV7131R */
#define USB_UDIA_MICRODIA_PRODUCT_ID_627F	0x627f			/**< EEPROM */
// SN9C202:
#define USB_UDIA_MICRODIA_PRODUCT_ID_6280	0x6280			/**< SN9C202 + MI1300 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6282	0x6282			/**< SN9C202 + MI1310 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6283	0x6283			/**< SN9C202 + S5K4AAFX */
#define USB_UDIA_MICRODIA_PRODUCT_ID_6288	0x6288			/**< SN9C202 + OV9655 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_628A	0x628a			/**< SN9C202 + ICM107 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_628B	0x628b			/**< SN9C202 + CX1332 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_628C	0x628c			/**< SN9C202 + MI1320 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_628E	0x628e			/**< SN9C202 + SOI968 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_628F	0x628f			/**< SN9C202 + OV9650 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_62A0	0x62a0			/**< SN9C202 + OV7670ISP */
#define USB_UDIA_MICRODIA_PRODUCT_ID_62A2	0x62a2			/**< SN9C202 + OM6802 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_62B0	0x62b0			/**< SN9C202 + MI0360/MT9V111 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_62B3	0x62b3			/**< SN9C202 + OV9655 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_62BA	0x62ba			/**< SN9C202 + S5K53BEB */
#define USB_UDIA_MICRODIA_PRODUCT_ID_62BB	0x62bb			/**< SN9C202 + OV7660 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_62BC	0x62bc			/**< SN9C202 + HV7131R */
#define USB_UDIA_MICRODIA_PRODUCT_ID_62BE	0x62be			/**< SN9C202 + OV7663 */
// SN9C325:
#define USB_UDIA_MICRODIA_PRODUCT_ID_6128	0x6128			/**< SN9C325 + OM6802 */
#define USB_UDIA_MICRODIA_PRODUCT_ID_612A	0x612a			/**< SN9C325 + OV7648 + POx1030xC + SOI768 + PO2030N + OV7660 + OV7670 + HV7131R */
// *** "Clone"-devices ***:
// Microsoft LifeCam VX-6000:
#define USB_UDIA_MICROSOFT_PRODUCT_ID_00F4	0x00f4			/**< => 628f (SN9C202 + OV9650) */
// Trust WB-3600R:
#define USB_UDIA_TRUST_PRODUCT_ID_013D		0x013d			/**< => 627b (SN9C201 + OV7660) */
// Chicony Panda 7, Trust WB-3450P:
#define USB_UDIA_CHICONY_PRODUCT_ID_A128	0xa128			/**< => 62be (SN9C202 + OV7663) + EEPROM */



/**
 * @def VID_HARDWARE_MICRODIA
 *
 * This value must be inserted into the kernel headers linux/videodev.h
 * It's useful only for the support of V4L v1
 */
#define VID_HARDWARE_MICRODIA		88




/**
 * @def MAX_ISO_BUFS
 *   Number maximal of ISOC buffers
 *
 * @def ISO_FRAMES_PER_DESC
 *   Number frames per ISOC descriptor
 */
#define MAX_ISO_BUFS				200
#define ISO_FRAMES_PER_DESC			10

/**
 * @def hb_multiplier(wMaxPacketSize)
 *   USB endpoint high bandwidth multiplier
 *
 * @def max_packet_sz(wMaxPacketSize)
 *   USB endpoint maximum packet size
 *
 * These values are both encoded within the wMaxPacketSize field of the usb_endpoint_descriptor structure.
 * The 11(10:0) lowest bits hold the MaxPacketSize(according to the usb specs a value over 1024 is undefined), 
 * and the bits 12:11 will give the high bandwidth multiplier(this should be a value of 0-2).
 */

#define hb_multiplier(wMaxPacketSize) (((wMaxPacketSize >> 11) & 0x03) + 1)
#define max_packet_sz(wMaxPacketSize) ((wMaxPacketSize) & 0x07ff)

/**
 * @def MICRODIA_MAX_IMAGES
 *   Absolute maximum number of buffers available for mmap()
 *
 * @def MICRODIA_FRAME_SIZE
 *   Maximum size after decompression
 */
#define MICRODIA_MAX_IMAGES			10
#define MICRODIA_FRAME_SIZE			(1280 * 1024 * 4)

/**
 * @def MICRODIA_FRAME_SIZE_DIVISOR_DEFAULT
 * 	Most models have a Frame size of image_size + image_size/2
 *
 * @def MICRODIA_FRAME_SIZE_DIVISOR_6260
 * 	The 6260 has a Frame size of image_size + image_size
 */
#define MICRODIA_FRAME_SIZE_DIVISOR_DEFAULT	2
#define MICRODIA_FRAME_SIZE_DIVISOR_6260	1





/**
 * @def DRIVER_SUPPORT
 *
 * List of supported devices
 */
#define DRIVER_SUPPORT				"Microdia USB Camera"

/**
 * @def CONFIG_MICRODIA_DEBUG
 *   Enable / Disable the debug mode.
 *
 * @def UDIA_INFO(str, args...)
 *   Print information message. 
 *   @a Use this function like the function printf.
 *
 * @def UDIA_ERROR(str, args...)
 *   Print error message.
 *   @a Use this function like the function printf.
 *
 * @def UDIA_WARNING(str, args...)
 *   Print warning message.
 *   @a Use this function like the function printf.
 *
 * @def UDIA_DEBUG(str, args...)
 *   Print debug message.
 *   @a Use this function like the function printf.
 */
#ifndef CONFIG_MICRODIA_DEBUG
#define CONFIG_MICRODIA_DEBUG 			0
#endif

#if CONFIG_MICRODIA_DEBUG

#define UDIA_INFO(str, args...)			printk(KERN_INFO PREFIX str, ##args)
#define UDIA_ERROR(str, args...)			printk(KERN_ERR PREFIX str, ##args)
#define UDIA_WARNING(str, args...)		printk(KERN_WARNING PREFIX str, ##args)
#define UDIA_DEBUG(str, args...)			printk(KERN_DEBUG PREFIX str, ##args)

#else

#define UDIA_INFO(str, args...)			printk(KERN_INFO PREFIX str, ##args)
#define UDIA_ERROR(str, args...)			printk(KERN_ERR PREFIX str, ##args)
#define UDIA_WARNING(str, args...)		printk(KERN_WARNING PREFIX str, ##args)
#define UDIA_DEBUG(str, args...)			do { } while(0)

#endif


/**
 * @def CONFIG_MICRODIA_DEBUG_STREAM
 *   Enable / Disable the debug mode about the stream.
 *
 * @def UDIA_STREAM(str, args...)
 *   Print stream debug message. 
 *   @a Use this function like the function printf.
 */
#ifndef CONFIG_MICRODIA_DEBUG_STREAM
#define CONFIG_MICRODIA_DEBUG_STREAM		0
#endif

#if CONFIG_MICRODIA_DEBUG_STREAM

#define UDIA_STREAM(str, args...)		printk(KERN_DEBUG PREFIX str, ##args)

#else

#define UDIA_STREAM(str, args...)		do { } while(0)

#endif



/**
 * @enum T_MICRODIA_DEVICE Video camera supported by the driver
 */
typedef enum {
	MICRODIA_6240 = 1,
	MICRODIA_6242 = 2,
	MICRODIA_6243 = 3,
	MICRODIA_6248 = 4,
	MICRODIA_624B = 5,
	MICRODIA_624C = 6,
	MICRODIA_624E = 7,
	MICRODIA_624F = 8,
	MICRODIA_6253 = 9,
	MICRODIA_6260 = 10,
	MICRODIA_6262 = 11,
	MICRODIA_6270 = 12,
	MICRODIA_627A = 13,
	MICRODIA_627B = 14,
	MICRODIA_627C = 15,
	MICRODIA_627F = 16,
	MICRODIA_6280 = 17,
	MICRODIA_6282 = 18,
	MICRODIA_6283 = 19,
	MICRODIA_6288 = 20,
	MICRODIA_628A = 21,
	MICRODIA_628B = 22,
	MICRODIA_628C = 23,
	MICRODIA_628E = 24,
	MICRODIA_628F = 25,
	MICRODIA_62A0 = 26,
	MICRODIA_62A2 = 27,
	MICRODIA_62B0 = 28,
	MICRODIA_62B3 = 29,
	MICRODIA_62BA = 30,
	MICRODIA_62BB = 31,
	MICRODIA_62BC = 32,
	MICRODIA_62BE = 33,
	MICRODIA_6128 = 34,
	MICRODIA_612A = 35
	// DO NOT ADD CLONE-DEVICES HERE !
} T_MICRODIA_DEVICE;


/**
 * @enum T_MICRODIA_VIDEOMODE Video feature supported by camera
 */
typedef enum {
	MICRODIA_VGA,						/**< For VGA video camera */
	MICRODIA_SXGA,						/**< For SXGA video camera 1.3M */
	MICRODIA_UXGA						/**< For UXGA video camera 2M */
} T_MICRODIA_VIDEOMODE;


/** 
 * @enum T_MICRODIA_RESOLUTION Video resolution
 */
typedef enum {
	MICRODIA_80x60,
	/*MICRODIA_128x96,*/
	MICRODIA_160x120,
	/*MICRODIA_213x160, */
	MICRODIA_320x240,
	MICRODIA_640x480,
	MICRODIA_800x600,
	MICRODIA_1024x768,
	MICRODIA_1280x1024,
	MICRODIA_NBR_SIZES
} T_MICRODIA_RESOLUTION;


/**
 * @enum T_MICRODIA_PALETTE Color palette
 * 	Change values to form a bitmap, to allow webcam model pallete selection
 */
typedef enum {
	MICRODIA_PALETTE_I420	= 0x0001,
	MICRODIA_PALETTE_RGB24	= 0x0002,
	MICRODIA_PALETTE_RGB32	= 0x0004,
	MICRODIA_PALETTE_BGR24	= 0x0008,
	MICRODIA_PALETTE_BGR32	= 0x0010,
	MICRODIA_PALETTE_UYVY	= 0x0020,
	MICRODIA_PALETTE_YUYV	= 0x0040,
	MICRODIA_PALETTE_YUV422	= 0x0080,
} T_MICRODIA_PALETTE;

/**
 * @static const microdia_supported_palettes[<cam_model]
 * 	Array of bitmap specifying which palettes are supported by a webcam.
 * 	Initially set so that all webcam except 6260 support
 * 	I420, RGB24 & BGR24.
 */
static const unsigned microdia_supported_palettes[] = {
	0,
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6240 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6242 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6243 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6248 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 624B */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 624C */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 624E */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 624F */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6253 */
	MICRODIA_PALETTE_YUV422,	/* 6260 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6262 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6270 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 627A */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 627B */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 627C */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 627F */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6280 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6282 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6283 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6288 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 628A */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 628B */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 628C */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 628E */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 628F */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 62A0 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 62A2 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 62B0 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 62B3 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 62BA */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 62BB */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 62BC */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 62BE */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 6128 */
	MICRODIA_PALETTE_I420 | MICRODIA_PALETTE_RGB24 |
		MICRODIA_PALETTE_BGR24,	/* 612A */
};

/**
 * @static const microdia_initial_palette[<cam_model]
 * 	Array specifying which palette is initially selected for a webcam.
 * 	Initially set so that all webcam except 6260 select I420.
 */
static const unsigned microdia_initial_palette[] = {
	0,
	MICRODIA_PALETTE_I420,	/* 6240 */
	MICRODIA_PALETTE_I420,	/* 6242 */
	MICRODIA_PALETTE_I420,	/* 6243 */
	MICRODIA_PALETTE_I420,	/* 6248 */
	MICRODIA_PALETTE_I420,	/* 624B */
	MICRODIA_PALETTE_I420,	/* 624C */
	MICRODIA_PALETTE_I420,	/* 624E */
	MICRODIA_PALETTE_I420,	/* 624F */
	MICRODIA_PALETTE_I420,	/* 6253 */
	MICRODIA_PALETTE_YUV422,	/* 6260 */
	MICRODIA_PALETTE_I420,	/* 6262 */
	MICRODIA_PALETTE_I420,	/* 6270 */
	MICRODIA_PALETTE_I420,	/* 627A */
	MICRODIA_PALETTE_I420,	/* 627B */
	MICRODIA_PALETTE_I420,	/* 627C */
	MICRODIA_PALETTE_I420,	/* 627F */
	MICRODIA_PALETTE_I420,	/* 6280 */
	MICRODIA_PALETTE_I420,	/* 6282 */
	MICRODIA_PALETTE_I420,	/* 6283 */
	MICRODIA_PALETTE_I420,	/* 6288 */
	MICRODIA_PALETTE_I420,	/* 628A */
	MICRODIA_PALETTE_I420,	/* 628B */
	MICRODIA_PALETTE_I420,	/* 628C */
	MICRODIA_PALETTE_I420,	/* 628E */
	MICRODIA_PALETTE_I420,	/* 628F */
	MICRODIA_PALETTE_I420,	/* 62A0 */
	MICRODIA_PALETTE_I420,	/* 62A2 */
	MICRODIA_PALETTE_I420,	/* 62B0 */
	MICRODIA_PALETTE_I420,	/* 62B3 */
	MICRODIA_PALETTE_I420,	/* 62BA */
	MICRODIA_PALETTE_I420,	/* 62BB */
	MICRODIA_PALETTE_I420,	/* 62BC */
	MICRODIA_PALETTE_I420,	/* 62BE */
	MICRODIA_PALETTE_I420,	/* 6128 */
	MICRODIA_PALETTE_I420,	/* 612A */
	// DO NOT ADD CLONE-DEVICES HERE !
};

/**
 * @struct microdia_iso_buf
 */
struct microdia_iso_buf {
	void *data;
	int length;
	int read;
	struct urb *urb;
};


/**
 * @struct microdia_frame_buf
 */
struct microdia_frame_buf {
	int errors;
	void *data;
	volatile int filled;
	struct microdia_frame_buf *next;
};


/**
 * @struct microdia_image_buf
 */
struct microdia_image_buf {
	unsigned long offset;				/**< Memory offset */
	int vma_use_count;					/**< VMA counter */
};


/**
 * @struct microdia_coord
 */
struct microdia_coord {
	int x;								/**< X-coordonate */
	int y;								/**< Y-coordonate */
};


/**
 * @struct microdia_video
 */
struct microdia_video {
	int fps;						/**< FPS setting */
	int brightness;						/**< Brightness setting */
	int contrast;						/**< Contrast setting */
	int whiteness;						/**< Whiteness setting */
	int colour;						/**< Colour setting */
	int depth;						/**< Depth colour setting */
	int palette;						/**< Palette setting */
	int hue;						/**< Hue setting */
	int hflip;						/**< Horizontal flip */
	int vflip;						/**< Vertical flip */
	int exposure;						/**< Exposure */
	int sharpness;						/**< Sharpness */
	int auto_exposure;					/**< Automatic exposure */
	int auto_whitebalance;					/**< Automatic whitebalance */
	int reg;						/**< register adress */
	int val;						/**< register value */
	int i2creg;						/**< I2C register adress */
	int i2cval;						/**< I2C register value */
	int i2cslave;						/**< I2C slave adress */
};

/**
 * @struct usb_microdia
 */
struct usb_microdia {
	struct video_device *vdev; 			/**< Pointer on a V4L2 video device */
	struct usb_device *udev;			/**< Pointer on a USB device */
	struct usb_interface *interface;	/**< Pointer on a USB interface */

	int release;						/**< Release of the device (bcdDevice) */
	int webcam_model;					/**< Model of video camera device */
	int webcam_type;					/**< Type of camera : VGA, SXGA (1.3M), UXGA (2M) */
	int frame_size_divisor;			/**> Used to calculate frame_size	 */

	__u8 sensor_slave_address;		/**< I2C/SCCB slave address of the image sensor */
	__u8 sensor_flags;			/**< I2C/SCCB flags of the image sensor */

	unsigned char *int_in_buffer;		/**< Interrupt IN buffer */
	size_t int_in_size;					/**< Interrupt IN buffer size */
	__u8 int_in_endpointAddr;			/**< Interrupt IN endpoint address */

	size_t isoc_in_size;				/**< Isochrone IN size */
	__u8 isoc_in_endpointAddr;			/**< Isochrone IN endpoint address */

	int watchdog;						/**< Counter for the software watchdog */

	struct microdia_video vsettings;		/**< Video settings (brightness, whiteness...) */

	int error_status;

	int vopen;							/**< Video status (Opened or Closed) */
	int visoc_errors;					/**< Count the number of ISOCH errors */
	int vframes_error;					/**< Count the number of fault frames (so dropped) */
	int vframes_dumped;					/**< Count the number of ignored frames */


	spinlock_t spinlock;				/**< Spin lock */
	struct semaphore mutex;				/**< Mutex */
	wait_queue_head_t wait_frame;		/**< Queue head */


	// 1: isoc
	char isoc_init_ok;
	struct microdia_iso_buf isobuf[MAX_ISO_BUFS];

	// 2: frame
	int frame_size;
	struct microdia_frame_buf *framebuf;
	struct microdia_frame_buf *empty_frames, *empty_frames_tail;
	struct microdia_frame_buf *full_frames, *full_frames_tail;
	struct microdia_frame_buf *fill_frame;
	struct microdia_frame_buf *read_frame;

	// 3: image
	int view_size;
	int image_size;
	void *image_data;
	struct microdia_image_buf images[MICRODIA_MAX_IMAGES];
	int image_used[MICRODIA_MAX_IMAGES];
	unsigned int nbuffers;
	unsigned int len_per_image;
	int image_read_pos;
	int fill_image;
	int resolution;
	struct microdia_coord view;
	struct microdia_coord image;

	int (* initialize) (struct usb_microdia *dev);
	int (* start_stream) (struct usb_microdia *dev);
	int (* stop_stream) (struct usb_microdia *dev);
	int (* set_contrast) (struct usb_microdia *dev);
	int (* set_brightness) (struct usb_microdia *dev);
	int (* set_gamma) (struct usb_microdia *dev);
	int (* set_exposure) (struct usb_microdia *dev);
	int (* flip_detect) (struct usb_microdia *dev);
	int (* set_hvflip) (struct usb_microdia *dev);
	int (* set_sharpness) (struct usb_microdia *dev);
	int (* set_auto_exposure) (struct usb_microdia *dev);
	int (* set_auto_whitebalance) (struct usb_microdia *dev);
};


/**
 * @def MICRODIA_PERCENT
 *   Calculate a value from a percent
 */
#define MICRODIA_PERCENT(x,y) ( ((int)x * (int)y) / 100)


/**
 * @def to_microdia_dev(d)
 * Cast a member of a structure out to the containing structure
 */
#define to_microdia_dev(d) container_of(d, struct usb_microdia, kref)


extern const struct microdia_coord microdia_image_sizes[MICRODIA_NBR_SIZES];

	
int usb_microdia_control_write(struct usb_microdia *, __u16, __u8 *, __u16);
int usb_microdia_control_write_multi(struct usb_microdia *, __u16*, __u8 *,
	__u16, __u16);
int usb_microdia_control_read(struct usb_microdia *, __u16, __u8 *, __u16);
int usb_microdia_set_feature(struct usb_microdia *, int);
int usb_microdia_set_configuration(struct usb_microdia *);
int usb_microdia_isoc_init(struct usb_microdia *);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
void usb_microdia_isoc_handler(struct urb *, struct pt_regs *);
#else
void usb_microdia_isoc_handler(struct urb *);
#endif
void usb_microdia_isoc_cleanup(struct usb_microdia *);

int dev_microdia_initialize_device(struct usb_microdia *);
int dev_microdia_start_stream(struct usb_microdia *);
int dev_microdia_stop_stream(struct usb_microdia *);
int dev_microdia_check_device(struct usb_microdia *, int);
int dev_microdia_camera_on(struct usb_microdia *);
int dev_microdia_camera_off(struct usb_microdia *);
int dev_microdia_camera_asleep(struct usb_microdia *);
int dev_microdia_init_camera(struct usb_microdia *);
int dev_microdia_reconf_camera(struct usb_microdia *);
int dev_microdia_camera_settings(struct usb_microdia *);
int dev_microdia_camera_set_contrast(struct usb_microdia *);
int dev_microdia_camera_set_brightness(struct usb_microdia *);
int dev_microdia_camera_set_gamma(struct usb_microdia *);
int dev_microdia_camera_set_exposure(struct usb_microdia *);
int dev_microdia_camera_set_hvflip(struct usb_microdia *);
int dev_microdia_camera_set_sharpness(struct usb_microdia *dev);
int dev_microdia_camera_set_auto_exposure(struct usb_microdia *dev);
int dev_microdia_camera_set_auto_whitebalance(struct usb_microdia *dev);
//int dev_microdia_set_camera_quality(struct usb_microdia *);
int dev_microdia_set_camera_fps(struct usb_microdia *);
int dev_microdia_watchdog_camera(struct usb_microdia *);

int v4l_microdia_select_video_mode(struct usb_microdia *, int, int);
int v4l_microdia_register_video_device(struct usb_microdia *);
int v4l_microdia_unregister_video_device(struct usb_microdia *);

int microdia_create_sysfs_files(struct video_device *);
void microdia_remove_sysfs_files(struct video_device *);

int microdia_allocate_buffers(struct usb_microdia *);
int microdia_reset_buffers(struct usb_microdia *);
int microdia_clear_buffers(struct usb_microdia *);
int microdia_free_buffers(struct usb_microdia *);
void microdia_next_image(struct usb_microdia *);
int microdia_next_frame(struct usb_microdia *);
int microdia_handle_frame(struct usb_microdia *);

int microdia_decompress(struct usb_microdia *);


#endif 
